#!/usr/bin/env groovy

import iControl.CommonStatistic;
import iControl.LocalLBNodeAddressNodeAddressStatisticEntry;
import iControl.LocalLBNodeAddressNodeAddressStatistics;

import com.urbancode.air.AirPluginTool

final def apTool = new AirPluginTool(this.args[0], this.args[1])
def m_interfaces = new iControl.Interfaces();
final def props = apTool.getStepProperties()

final def serverUrlArray = props['serverURL']?.split('://') as String[]
final def username = props['username']
final def password = props['password']
final def nodeAddress = props['nodeAddress'] as String
final def doWait = Boolean.valueOf(props['doWaitForConnectionClose'])
final def sleepInterval = (props['sleepInterval'])?Long.valueOf(props['sleepInterval']):5
final def timeout = Integer.parseInt(props['timeout'])

def getActiveConnections = { nodeAddressBinding ->        
    def stats = nodeAddressBinding.get_all_statistics();
    def statsEntries = stats.getStatistics();
    for (statsEntry in statsEntries) {
        String nodeAddressToTest = statsEntry.getNode_address();
        if (nodeAddressToTest.equals(nodeAddress)){
            CommonStatistic[] comStats = statsEntry.getStatistics();
            for(CommonStatistic comStat : comStats) {
                if (comStat.getType().equals(iControl.CommonStatisticType.STATISTIC_SERVER_SIDE_CURRENT_CONNECTIONS)) {
                    return comStat.getValue().getLow();
                }
            }
        }
    }
}
def waitForConnections = { nodeAddressBinding ->
    def currentConnections = getActiveConnections(nodeAddressBinding)
    while(currentConnections > 0) {
        println("Waiting for $currentConnections to close, sleep for $sleepInterval second(s)")
        sleep(sleepInterval * 1000, {println("Waiting aborted!"); return true})
        currentConnections = getActiveConnections(nodeAddressBinding)
    }
    println("All connections closed for pool, done waiting.")
}

try {
    def connectionString = serverUrlArray.length > 1? serverUrlArray[1] : serverUrlArray[0]
    if (connectionString.endsWith('/'))
    connectionString = connectionString.substring(0, connectionString.length() - 1)

    //m_interfaces_initialize must be called to use its methods
    m_interfaces.initialize(connectionString, username, password)
    def nodeAddressBinding = m_interfaces.getLocalLBNodeAddress()
    nodeAddressBinding.setTimeout(timeout * 1000)
    if (doWait) {
        waitForConnections(nodeAddressBinding)
    }
    String[] nodeAddressArray = [nodeAddress]
    iControl.CommonEnabledState[] disabledStatus = [iControl.CommonEnabledState.STATE_DISABLED]
    nodeAddressBinding.set_session_enabled_state(nodeAddressArray, disabledStatus)
    println "Disabled node ${nodeAddress} across all pools and ports"
    
}
catch (Exception e) {
    e.printStackTrace()
    System.exit(1)
}
